/*
 * [The "BSD licence"]
 * Copyright (c) 2012 DataTables4j
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 3. Neither the name of DataTables4j nor the names of its contributors 
 * may be used to endorse or promote products derived from this software 
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.github.datatables4j.plugins.ajax;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.codehaus.jackson.JsonNode;
import org.codehaus.jackson.JsonParseException;
import org.codehaus.jackson.map.JsonMappingException;
import org.codehaus.jackson.map.ObjectMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.github.datatables4j.core.api.constants.DTConstants;
import com.github.datatables4j.core.api.datasource.AjaxSource;
import com.github.datatables4j.core.api.exception.DataNotFoundException;
import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.WebResource;

/**
 * Jersey implementation of DataProvider for AJAX table.
 * 
 * @author Thibault Duchateau
 */
public class JerseyDataProvider implements AjaxSource {

	// Logger
	private static Logger logger = LoggerFactory.getLogger(JerseyDataProvider.class);
		
	/**
	 * TODO
	 */
	public Object getData(String wsUrl) throws DataNotFoundException {

		logger.debug("Retrieving data with JerseyDataProvider ...");
		
		// Jersey client initialisation
		Client client = Client.create();

		// WebResource initialisation
		WebResource webResource = client.resource(wsUrl);
		
		logger.debug("Web service call ...");
		
		// Performing GET request
		ClientResponse response = webResource.accept("application/json").get(
				ClientResponse.class);
		if (response.getStatus() != 200) {
			throw new DataNotFoundException("No data found : " + String.valueOf(response.getStatus()));
		}
		
		String output = response.getEntity(String.class);
		
		// Jackson object mapper
		ObjectMapper mapper = new ObjectMapper();

		Map<String, Object> map = new HashMap<String, Object>();
		JsonNode df;

		try {
			logger.debug("Converting response to JSON ...");
			df = mapper.readValue(output, JsonNode.class);
			
			logger.debug("Convertion OK");
			map.put(DTConstants.DT_DS_DATA, df);
			
		} catch (JsonParseException e) {
			throw new DataNotFoundException(e);
		} catch (JsonMappingException e) {
			throw new DataNotFoundException(e);
		} catch (IOException e) {
			throw new DataNotFoundException(e);
		}

		return df;
	}
}